import json

from dateutil.parser import parse as parse_date

from django.contrib.contenttypes.models import ContentType
from django.core.cache import cache
from django.http import Http404
from django.utils import timezone, dateparse

from rest_framework import viewsets
from rest_framework.permissions import IsAdminUser
from rest_framework.response import Response
from rest_framework.views import APIView

from bulbs.api.permissions import CanPromoteContent
from bulbs.api.mixins import UncachedResponse

from bulbs.utils.methods import get_query_params

from .models import PZone
from .operations import PZoneOperation, InsertOperation, DeleteOperation, ReplaceOperation
from .serializers import PZoneSerializer, InsertOperationSerializer, DeleteOperationSerializer, ReplaceOperationSerializer


class OperationsViewSet(APIView):

    permission_classes = [IsAdminUser, CanPromoteContent]

    def get_serializer_class_by_name(self, type_name):
        try:
            # try to convert type name into a serializer class
            app_label, model_name = type_name.split("_")
            operation_type = ContentType.objects.get_by_natural_key(app_label, model_name)
            operation_model = operation_type.model_class()
            return self.get_serializer_class(operation_model)
        except (ValueError, ContentType.DoesNotExist):
            # invalid type name
            raise ContentType.DoesNotExist("Provided type_name is invalid.")

    def get_serializer_class(self, obj_class):
        serializer_class = None
        if obj_class is InsertOperation:
            serializer_class = InsertOperationSerializer
        elif obj_class is DeleteOperation:
            serializer_class = DeleteOperationSerializer
        elif obj_class is ReplaceOperation:
            serializer_class = ReplaceOperationSerializer
        return serializer_class

    def serialize_operations(self, operations):
        """Serialize a list of operations into JSON."""

        serialized_ops = []
        for operation in operations:
            serializer = self.get_serializer_class(operation.__class__)
            serialized_ops.append(serializer(operation).data)
        return serialized_ops

    def get(self, request, pzone_pk):
        """Get all the operations for a given pzone."""

        # attempt to get given pzone
        try:
            pzone = PZone.objects.get(pk=pzone_pk)
        except PZone.DoesNotExist:
            raise Http404("Cannot find given pzone.")

        # bulid filters
        filters = {"pzone": pzone}

        if "from" in request.GET:
            parsed = dateparse.parse_datetime(request.GET["from"])
            if parsed is not None:
                filters["when__gte"] = parsed

        if "to" in request.GET:
            parsed = dateparse.parse_datetime(request.GET["to"])
            if parsed is not None:
                filters["when__lt"] = parsed

        # get operations and serialize them
        operations = PZoneOperation.objects.filter(**filters)

        # return a json response with serialized operations
        return Response(self.serialize_operations(operations), content_type="application/json")

    def post(self, request, pzone_pk):
        """Add a new operation to the given pzone, return json of the new operation."""

        # attempt to get given content list
        pzone = None
        try:
            pzone = PZone.objects.get(pk=pzone_pk)
        except PZone.DoesNotExist:
            raise Http404("Cannot find given pzone.")

        json_obj = []
        http_status = 500

        json_op = json.loads(request.body.decode("utf8"))
        if not isinstance(json_op, list):
            json_op = [json_op]

        for data in json_op:
            try:
                serializer = self.get_serializer_class_by_name(data["type_name"])
            except ContentType.DoesNotExist as e:
                json_obj = {"errors": [str(e)]}
                http_status = 400
                break

            serialized = serializer(data=data)
            if serialized.is_valid():
                # object is valid, save it
                serialized.save()

                # set response data
                json_obj.append(serialized.data)
                http_status = 200
            else:
                # object is not valid, return errors in a 400 response
                json_obj = serialized.errors
                http_status = 400
                break

        if http_status == 200 and len(json_obj) == 1:
            json_obj = json_obj[0]

        # cache the time in seconds until the next operation occurs
        next_ops = PZoneOperation.objects.filter(when__lte=timezone.now())
        if len(next_ops) > 0:
            # we have at least one operation, ordered soonest first
            next_op = next_ops[0]
            # cache with expiry number of seconds until op should exec
            cache.set('pzone-operation-expiry-' + pzone.name, next_op.when, 60 * 60 * 5)

        return Response(
            json_obj,
            status=http_status,
            content_type="application/json"
        )

    def delete(self, request, pzone_pk, operation_pk):
        """Remove an operation from the given pzone."""

        # note : we're not using the pzone_pk here since it's not actually
        #   necessary for getting an operation by pk, but it sure makes the urls
        #   nicer!

        # attempt to delete operation
        try:
            operation = PZoneOperation.objects.get(pk=operation_pk)
        except PZoneOperation.DoesNotExist:
            raise Http404("Cannot find given operation.")

        # delete operation
        operation.delete()

        # successful delete, return 204
        return Response("", 204)


class PZoneViewSet(UncachedResponse, viewsets.ModelViewSet):
    """Uncached viewset for `bulbs.promotions.PZone` model."""

    queryset = PZone.objects.all()
    serializer_class = PZoneSerializer
    paginate_by = 20
    permission_classes = [IsAdminUser, CanPromoteContent]

    def perform_update(self, serializer):
        """creates a record in the `bulbs.promotion.PZoneHistory`

        :param obj: the instance saved
        :param created: boolean expressing if the object was newly created (`False` if updated)
        """
        instance = serializer.save()
        # create history object
        instance.history.create(data=instance.data)

    def retrieve(self, request, *args, **kwargs):
        """Retrieve pzone as a preview or applied if no preview is provided."""

        when_param = get_query_params(self.request).get("preview", None)
        pk = self.kwargs["pk"]

        when = None
        if when_param:
            try:
                when = parse_date(when_param)
            except ValueError:
                # invalid format, set back to None
                when = None

        pzone = None
        if when:
            # we have a date, use it
            pzone = PZone.objects.preview(pk=pk, when=when)
        else:
            # we have no date, just get the pzone
            pzone = PZone.objects.applied(pk=pk)

        # turn content list into json
        return Response(PZoneSerializer(pzone).data, content_type="application/json")



from django.utils.dateparse import parse_datetime


class PromotionMiddleware(object):

    def process_template_response(self, request, response):
        when = None
        if request.method == "GET" and "pzone_preview" in request.GET:
            when = parse_datetime(request.GET["pzone_preview"])

        if when:
            response.context_data["pzone_preview"] = when

        return response



from django.core.management.base import BaseCommand
from django.utils import timezone

from ...models import SpecialCoverage


class Command(BaseCommand):
    help = "Migrates Special Coverages to published"

    def get_month_start_date(self):
        """Returns the first day of the current month"""
        now = timezone.now()
        return timezone.datetime(day=1, month=now.month, year=now.year, tzinfo=now.tzinfo)

    def get_absurd_end_date(self):
        """
        To give ourselves a buffer, we are setting the initial end_date to 5 years in the future.
        """
        now = timezone.now()
        return timezone.datetime(day=1, month=now.month, year=now.year + 5, tzinfo=now.tzinfo)

    def get_active_special_coverages(self):
        return SpecialCoverage.objects.filter(active=True)

    def update_special_coverage_instance(self, instance, **kwargs):
        for attr, value in kwargs.items():
            if not hasattr(instance, attr):
                raise AttributeError(
                    """{0} has no attribute {1}.""".format(instance._meta.model.__name__, attr)
                )
            setattr(instance, attr, value)
        instance.save()
        return instance

    def publish_active_special_coverages(self):
        start = self.get_month_start_date()
        end = self.get_absurd_end_date()
        for sc in self.get_active_special_coverages():
            active = getattr(sc, "active", None)
            if active is None:
                raise AttributeError(
                    """Special Coverage no longer has an active attribute."""
                )
            self.update_special_coverage_instance(sc, start_date=start, end_date=end)

    def handle(self, *args, **kwargs):
        self.publish_active_special_coverages()



class TagCache(object):
    """
    in-memory cache
    """

    _cache = {}  # Maybe too terrible?

    @classmethod
    def count(cls, slug):
        """get the number of objects in the cache for a given slug

        :param slug: cache key
        :return: `int`
        """
        from .models import Content
        # Gets the count for a tag, hopefully form an in-memory cache.
        cnt = cls._cache.get(slug)
        if cnt is None:
            cnt = Content.search_objects.search(tags=[slug]).count()
            cls._cache[slug] = cnt
        return cnt



try:
    from urllib.parse import urljoin
except ImportError:
    from urlparse import urljoin

import requests

from django.conf import settings


class RequestFailure(requests.exceptions.RequestException):
    """The request failed."""


class TunicClient(object):

    def get(self, path):
        headers = {
            'Authorization': 'Token {}'.format(settings.TUNIC_REQUEST_TOKEN),
        }

        url_base = settings.TUNIC_STAFF_BACKEND_ROOT
        # Handle protocol-relative root paths
        if url_base.startswith('//'):
            url_base = 'http:' + url_base

        url = urljoin(url_base,
                      settings.TUNIC_API_PATH.rstrip('/') + '/' + path.lstrip('/'))

        resp = requests.get(url, headers=headers)
        if not resp.ok:
            raise RequestFailure(response=resp)
        return resp

    def get_active_campaigns(self):
        return {c['id']: c
                for c in self.get('/campaign/?active=true').json().get('results', {})}



from celery import shared_task


@shared_task(default_retry_delay=5)
def save_campaign_special_coverage_percolator(campaign_pk):
    from bulbs.special_coverage.models import SpecialCoverage
    for special_coverage in SpecialCoverage.objects.filter(campaign_id=campaign_pk):
        special_coverage._save_percolator()



import logging
import pytz
import requests

from django.conf import settings
from django.db import models

from djbetty import ImageField

from bulbs.utils import vault
from bulbs.poll.sodahead import (
    SodaheadResponseError, SodaheadResponseFailure, BLANK_ANSWER, DEFAULT_ANSWER_1,
    DEFAULT_ANSWER_2, SODAHEAD_DATE_FORMAT, SODAHEAD_POLL_ENDPOINT, SODAHEAD_POLLS_ENDPOINT,
    SODAHEAD_DELETE_POLL_ENDPOINT
)
from .managers import PollManager


logger = logging.getLogger(__name__)


class PollMixin(models.Model):
    """Give a child object all Poll related fields and logic."""
    question_text = models.TextField(blank=True, default="")
    sodahead_id = models.CharField(max_length=20, blank=True, default="")
    last_answer_index = models.IntegerField(default=0)
    end_date = models.DateTimeField(null=True, default=None)
    poll_image = ImageField(null=True, blank=True)
    answer_type = models.TextField(blank=True, default="text")

    search_objects = PollManager()

    class Meta:
        abstract = True

    def get_sodahead_data(self):
        response = requests.get(SODAHEAD_POLL_ENDPOINT.format(self.sodahead_id))

        if not response.ok:
            logger.error(
                'Poll(id: %s, sodahead_id: %s).get_sodahead_data status_code: %s error: %s',
                self.id,
                self.sodahead_id,
                response.text,
                response.status_code,
            )
            return {
                'poll': {
                    'totalVotes': 0,
                    'answers': [],
                },
            }
        else:
            return response.json()

    def get_sodahead_token(self):
        return vault.read(settings.SODAHEAD_TOKEN_VAULT_PATH)['value']

    def sodahead_payload(self):
        payload = {
            'access_token': self.get_sodahead_token(),
            'name': self.title,
            'title': self.question_text,
        }

        if self.sodahead_id:
            payload['id'] = self.sodahead_id

        if self.published:
            activation_date = self.published.astimezone(pytz.utc)
            payload['activationDate'] = activation_date.strftime(SODAHEAD_DATE_FORMAT)
        else:
            payload['activationDate'] = None

        if self.end_date:
            end_date = self.end_date.astimezone(pytz.utc)
            payload['endDate'] = end_date.strftime(SODAHEAD_DATE_FORMAT)
        else:
            payload['endDate'] = ''

        for answer in self.answers.all():
            if answer.answer_text and answer.answer_text is not u'':
                payload[answer.sodahead_answer_id] = answer.answer_text
            else:
                payload[answer.sodahead_answer_id] = BLANK_ANSWER

        if 'answer_01' not in payload:
            payload['answer_01'] = DEFAULT_ANSWER_1

        if 'answer_02' not in payload:
            payload['answer_02'] = DEFAULT_ANSWER_2

        return payload

    def save(self, *args, **kwargs):
        if not self.sodahead_id:
            response = requests.post(SODAHEAD_POLLS_ENDPOINT, self.sodahead_payload())

            if response.ok:
                self.sodahead_id = response.json()['poll']['id']
            elif response.status_code > 499:
                raise SodaheadResponseError(response.text)
            else:
                raise SodaheadResponseFailure(response.text)
        else:
            self.sync_sodahead()

        super(PollMixin, self).save(*args, **kwargs)

    def delete(self, *args, **kwargs):
        response = requests.delete(
            SODAHEAD_DELETE_POLL_ENDPOINT.format(
                self.sodahead_id,
                self.get_sodahead_token(),
            )
        )
        if response.status_code > 499:
            raise SodaheadResponseError(response.text)
        elif response.status_code > 399:
            raise SodaheadResponseFailure(response.text)
        super(PollMixin, self).delete(*args, **kwargs)

    def sync_sodahead(self):
        response = requests.post(
            SODAHEAD_POLL_ENDPOINT.format(self.sodahead_id),
            self.sodahead_payload()
        )

        if response.status_code > 499:
            raise SodaheadResponseError(response.json())
        elif response.status_code > 399:
            raise SodaheadResponseFailure(response.json())


class AnswerMixin(models.Model):
    """Generic answer fields. Excludes the Poll relationship."""
    sodahead_answer_id = models.CharField(max_length=20, blank=True, default="")
    answer_text = models.TextField(blank=True, default="")
    answer_image = ImageField(null=True, blank=True)

    class Meta:
        abstract = True



from django.db import models

from djes.models import IndexableManager


class SectionIndexableManager(IndexableManager):

    def get_by_identifier(self, identifier):
        identifier_id = identifier.split(".")[-1]
        return self.get(id=identifier_id)


class SectionManager(models.Manager):

    def get_by_identifier(self, identifier):
        identifier_id = identifier.split(".")[-1]
        return self.get(id=identifier_id)



from requests import ConnectionError

from django.conf import settings

from elasticsearch_dsl import filter as es_filter
from pageview_client.clients import TrendingClient

from bulbs.content.models import Content


DEFAULT_LIMIT = 10


trending_client = TrendingClient(settings.DIGEST_HOSTNAME, settings.DIGEST_ENDPOINT)


def get_popular_ids(limit=DEFAULT_LIMIT):
    try:
        ids = trending_client.get(settings.DIGEST_SITE, offset=settings.DIGEST_OFFSET, limit=limit)
        return list(ids)[:limit]
    except ConnectionError:
        return None


def popular_content(**kwargs):
    """
    Use the get_popular_ids() to retrieve trending content objects.
    Return recent content on failure.
    """
    limit = kwargs.get("limit", DEFAULT_LIMIT)
    popular_ids = get_popular_ids(limit=limit)
    if not popular_ids:
        # Return most recent content
        return Content.search_objects.search().extra(size=limit)
    return Content.search_objects.search().filter(es_filter.Ids(values=popular_ids))



try:
    from urllib.parse import urljoin, urlencode
except ImportError:
    from urlparse import urljoin
    from urllib import urlencode

from django import template
from django.conf import settings


register = template.Library()


@register.simple_tag
def content_tunic_campaign_url(campaign_id,
                               image_ratio=None,
                               image_width=None,
                               image_format=None):

    url_base = "campaign/{}/public".format(campaign_id)

    raw_params = {}
    if image_ratio:
        raw_params["image_ratio"] = image_ratio
    if image_width:
        raw_params["image_width"] = image_width
    if image_format:
        raw_params["image_format"] = image_format
    url_params = "{}".format(urlencode(raw_params))

    path = urljoin(
        settings.TUNIC_API_PATH,
        "{}?{}".format(url_base, url_params)
    )
    return urljoin(settings.TUNIC_BACKEND_ROOT, path)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import json_field.fields


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0001_initial'),
        ('content', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ContentList',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.SlugField(unique=True)),
                ('length', models.IntegerField(default=10)),
                ('data', json_field.fields.JSONField(default=[], help_text='Enter a valid JSON object')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='ContentListHistory',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('data', json_field.fields.JSONField(default=[], help_text='Enter a valid JSON object')),
                ('date', models.DateTimeField(auto_now_add=True)),
                ('content_list', models.ForeignKey(related_name='history', to='promotion.ContentList')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='ContentListOperation',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('when', models.DateTimeField()),
                ('applied', models.BooleanField(default=False)),
            ],
            options={
                'ordering': ['-when'],
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='InsertOperation',
            fields=[
                ('contentlistoperation_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='promotion.ContentListOperation')),
                ('index', models.IntegerField(default=0)),
                ('lock', models.BooleanField(default=False)),
                ('content', models.ForeignKey(related_name='+', to='content.Content')),
            ],
            options={
                'abstract': False,
            },
            bases=('promotion.contentlistoperation',),
        ),
        migrations.CreateModel(
            name='LockOperation',
            fields=[
                ('contentlistoperation_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='promotion.ContentListOperation')),
                ('target', models.ForeignKey(related_name='+', to='content.Content')),
            ],
            options={
                'abstract': False,
            },
            bases=('promotion.contentlistoperation',),
        ),
        migrations.CreateModel(
            name='ReplaceOperation',
            fields=[
                ('contentlistoperation_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='promotion.ContentListOperation')),
                ('lock', models.BooleanField(default=False)),
                ('content', models.ForeignKey(related_name='+', to='content.Content')),
                ('target', models.ForeignKey(related_name='+', to='content.Content')),
            ],
            options={
                'abstract': False,
            },
            bases=('promotion.contentlistoperation',),
        ),
        migrations.CreateModel(
            name='UnlockOperation',
            fields=[
                ('contentlistoperation_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='promotion.ContentListOperation')),
                ('target', models.ForeignKey(related_name='+', to='content.Content')),
            ],
            options={
                'abstract': False,
            },
            bases=('promotion.contentlistoperation',),
        ),
        migrations.AddField(
            model_name='contentlistoperation',
            name='content_list',
            field=models.ForeignKey(related_name='operations', to='promotion.ContentList'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='contentlistoperation',
            name='polymorphic_ctype',
            field=models.ForeignKey(related_name='polymorphic_promotion.contentlistoperation_set', editable=False, to='contenttypes.ContentType', null=True),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import DataMigration
from django.db import models

class Migration(DataMigration):

    def forwards(self, orm):
        "Write your forwards methods here."
        PERM_CONF = {
            "publish_content": "Can publish content",
            "publish_own_content": "Can publish own content",
            "change_content": "Can change content",
            "promote_content": "Can promote content"
        }
        GROUP_CONF = dict(
            contributor=(),
            author=("publish_own_content",),
            editor=(
                "publish_content",
                "change_content",
                "promote_content",
            ),
            admin=(
                "publish_content",
                "change_content",
                "promote_content",
            )
        )
        content_ct, _ = orm["contenttypes.ContentType"].objects.get_or_create(
            model="content", app_label="content"
        )
        for group_name, group_perms in GROUP_CONF.items():
            group, _ = orm["auth.Group"].objects.get_or_create(
                name=group_name
            )
            for perm_name in group_perms:
                perm, _ = orm["auth.Permission"].objects.get_or_create(
                    content_type=content_ct,
                    codename=perm_name,
                    defaults={
                        "name": PERM_CONF[perm_name]
                    }
                )
                group.permissions.add(perm)

    def backwards(self, orm):
        "Write your backwards methods here."

    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'content.content': {
            'Meta': {'object_name': 'Content'},
            '_thumbnail': ('djbetty.fields.ImageField', [], {'default': 'None', 'null': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'default': "''", 'max_length': '1024', 'blank': 'True'}),
            'feature_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['content.FeatureType']", 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'indexed': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'last_modified': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'auto_now': 'True', 'blank': 'True'}),
            'polymorphic_ctype': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'polymorphic_content.content_set'", 'null': 'True', 'to': u"orm['contenttypes.ContentType']"}),
            'published': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'default': "''", 'max_length': '50', 'blank': 'True'}),
            'subhead': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '255', 'blank': 'True'}),
            'tags': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['content.Tag']", 'symmetrical': 'False', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '512'})
        },
        u'content.featuretype': {
            'Meta': {'object_name': 'FeatureType'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '50'})
        },
        u'content.tag': {
            'Meta': {'object_name': 'Tag'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'polymorphic_ctype': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'polymorphic_content.tag_set'", 'null': 'True', 'to': u"orm['contenttypes.ContentType']"}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '50'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        }
    }

    complete_apps = ['content']
    symmetrical = True

